#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"
#include "MetersIncludes.h"


#include "emac.h"
#include "partest.h"
#include "portlcd.h"
#include "flash.h"

extern xMetersSemaphore;
extern xTickCount; /* millisecond tick counter */


int globalValue = 0;

void handleMeterTick(portSHORT portNum);

/* The interrupt entry point. */
void vP2_0_ISR_Wrapper( void ) __attribute__((naked));

/* The function that actually performs the interrupt processing.  This must be 
separate to the wrapper to ensure the correct stack frame is set up. */
void vP2_0_ISR_Handler( void );

void vP2_0_ISR_Handler( void )
{
extern int testValue;
  portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

  testValue++;
  if ((IO2_INT_STAT_R & (1 << 10)) != 0) /* P2.10 interrupt triggered - SPI Load Switch */
  {
    handleMeterTick(METER_INDEX_TST);
    IO2_INT_CLR |= (1<<10);
  }
  if ((IO2_INT_STAT_R & (1 << 0)) != 0) /* P0.0 interrupt triggered - connector J4, Pin 0*/
  {
    handleMeterTick(METER_INDEX_GAS);
    IO2_INT_CLR |= (1<<0);
  }
  if ((IO2_INT_STAT_R & (1 << 1)) != 0) /* P0.1 interrupt triggered - connector J4, Pin 1*/
  {
    handleMeterTick(METER_INDEX_EL1);
    IO2_INT_CLR |= (1<<1);
  }
  if ((IO2_INT_STAT_R & (1 << 2)) != 0) /* P0.3 interrupt triggered - connector J4, Pin 3*/
  {
    handleMeterTick(METER_INDEX_EL2);
    IO2_INT_CLR |= (1<<2);
  }
  if ((IO2_INT_STAT_R & (1 << 3)) != 0) /* P0.4 interrupt triggered - connector J4, Pin 4*/
  {
    handleMeterTick(METER_INDEX_AU1);
    IO2_INT_CLR |= (1<<3);
  }
  // P0.4 is not connected!
  //if ((IO2_INT_STAT_R & (1 << 4)) != 0) /* P0.2 interrupt triggered - connector J4, Pin 2*/
  //{
  // handleMeterTick(METER_INDEX_AU2);
  //  IO2_INT_CLR |= (1<<4);
  //}
  /* Clear the interrupt. */ 
  VICVectAddr = 0;

  /* Unblock the Meters task as data has arrived. */
  xSemaphoreGiveFromISR( xMetersSemaphore, &xHigherPriorityTaskWoken );

  if( xHigherPriorityTaskWoken )
   {
      /* If the Meters task was unblocked then calling "Yield from ISR" here
      will ensure the interrupt returns directly to the Meters task, if it
      is the highest priority read task. */
        portYIELD_FROM_ISR();
   }
}
/*-----------------------------------------------------------*/

void vP2_0_ISR_Wrapper( void )
{
	/* Save the context of the interrupted task. */
	portSAVE_CONTEXT();

	/* Call the handler function.  This must be separate from the wrapper
	function to ensure the correct stack frame is set up. */
	vP2_0_ISR_Handler();

	/* Restore the context of whichever task is going to run next. */
	portRESTORE_CONTEXT();
}

void handleMeterTick(portSHORT portNum)
{
  unsigned long long currentTime = getEpochTimeWithMs(); /* current epoch time with ms in ms */

  if (portNum >= NUMBER_OF_METERS) /* allowed index: 0 to (NUMBER_OF_METERS-1) */
  {
    return; /* invalid value */
  }

  globalValue++;
    //   LCD_cls();
  //LCD_puts( ( signed portCHAR * ) "xyz" );
  

  if (  currentTime > (meterItems[portNum].timeLastTick + DEBOUNCE_TICK_THRESHOLD_MS) )
  {
    /* we have a tick that is not a bounce */
    meterItems[portNum].currentValue++; /* one more tick. always increase the counter */


    /* now check if we should accumulate the ticks for this time period so we leave the new data item available long
     * enough on the web page to be read by the server 
     */
    if ( currentTime > ((unsigned long long)meterItems[portNum].timeBeforeLastTick + ((unsigned long long)ACCUMULATE_TICK_THRESHOLD_SEC * (unsigned long long)1000)) )
    {


 

      /* if the last meter tick happened longer than DEBOUNCE_TICK_THRESHOLD_MS ago (i.e. the switch did not just bounce),
       * we create a new data item
       */
      meterItems[portNum].timeBeforeLastTick = meterItems[portNum].timeLastTick;     /* when did the tick before the last tick occur? (epoch in milliseconds) */
      meterItems[portNum].timeLastTick = currentTime; /* when did the last tick occur? (epoch in milliseconds) */
      meterItems[portNum].valueBeforeLastTick = meterItems[portNum].valueLastTick;   /* what value did we have at the tick before the last tick? */
      meterItems[portNum].valueLastTick = meterItems[portNum].currentValue;   /* what value did we have at the last tick? now one more! */
      /* at present, valueLastTick and currentValue are the same. This might be different for future versions. */
    }
    else
    {
      /* do not create a new data item, but update the current "last" data item */
      meterItems[portNum].timeLastTick = currentTime;
      meterItems[portNum].valueLastTick = meterItems[portNum].currentValue;
    }
  }
  else
  {
    ; /* we bounced, ignore this tick */
  }
}

